#!/bin/bash
# Parameter:
#   ($1 recordingdir)
#   ($2 title)
#   ($3 artist)
#   ($4 year)

TEMPPATH=/tmp
VDRSYNC=/usr/bin/vdrsync.pl
LAME=/usr/bin/lame
BITRATE=160
SAMPLERATE=48000
TARGET=/media/mp3

function die ()
{
  echo "Verwendung: $(basename $0) /video/MyRecording/DATE/"
  echo "  es erzeugt aus /video/MyRecording/DATE/00*.vdr => /video/MyRecording/DATE/MyRecording-DATE.mp3"
  if test "$1"; then
    echo "$1"
  fi
  exit 1
}

if ! test "$1"; then
  die "Aufnahmeverzeichnis nicht angegeben."
fi

if test -d "$1"; then
    AUFNAHME="$1"
else
    AUFNAHME=$(echo $(dirname "$1"))
fi

Date=$(echo $(basename "$AUFNAHME") | sed s/.rec//g;)
Recording=$(echo $(basename $(dirname "$AUFNAHME")))
Title="$Recording-$Date"
Title=$(echo "$Title" | sed -e "s/^_-//g;")

# extract mp2 stream
nice $VDRSYNC -cut -audio-only -o "$TEMPPATH" -basename "$Title-" "$AUFNAHME"

if ! test -e "$TEMPPATH/$Title-0.mpa"; then
 die "MP2 Stream $TEMPPATH/$Title-0.mpa wurde nicht demuxt!?"
fi

# Defaultparameter
LAMEPAR="-h -m s -b $BITRATE "
ID3TAGS=""

# Adjust Samplerate like 44100,48000 ...
if test $SAMPLERATE -ne 48000; then
  LAMEPAR="$LAMEPAR --resample $SAMPLERATE"
fi

mkdir $TARGET/$Title
$LAME $LAMEPAR --mp2input --add-id3v2 --tt "$2" --ta "$3" --ty "$4" "$TEMPPATH/$Title-0.mpa" "$TARGET/$Title/$3 - $2.mp3"

rm "$TEMPPATH/$Title-0.mpa"