#!/bin/bash
# Parameter:
#   ($1 recordingdir)
#   ($2 title)
#   ($3 artist)
#   ($4 year)

TEMPPATH=/tmp
VDRSYNC=/usr/bin/vdrsync.pl
MPG123=/usr/bin/mpg123
OGGENC=/usr/bin/oggenc
BITRATE=160
SAMPLERATE=48000
TARGET=/media/mp3

function die ()
{
  echo "Verwendung: $(basename $0) /video/MyRecording/DATE/"
  echo "  es erzeugt aus /video/MyRecording/DATE/00*.vdr => /video/MyRecording/DATE/MyRecording-DATE.ogg"
  if test "$1"; then
    echo "$1"
  fi
  exit 1
}

if ! test "$1"; then
  die "Aufnahmeverzeichnis nicht angegeben."
fi

if test -d "$1"; then
    AUFNAHME="$1"
else
    AUFNAHME=$(echo $(dirname "$1"))
fi

Date=$(echo $(basename "$AUFNAHME") | sed s/.rec//g;)
Recording=$(echo $(basename $(dirname "$AUFNAHME")))
Title="$Recording-$Date"
Title=$(echo "$Title" | sed -e "s/^_-//g;")

# extract mp2 stream
nice $VDRSYNC -cut -audio-only -o "$TEMPPATH" -basename "$Title-" "$AUFNAHME"

if ! test -e "$TEMPPATH/$Title-0.mpa"; then
 die "MP2 Stream $TEMPPATH/$Title-0.mpa wurde nicht demuxt!?"
fi

# Defaultparameter
OGGENCPAR="-q 3"
ID3TAGS=""

# Adjust Samplerate like 44100,48000 ...
if test $SAMPLERATE -ne 48000; then
  LAMEPAR="$OGGENCPAR --resample $SAMPLERATE"
fi

mkdir $TARGET/$Title

$MPG123 --wav - "$TEMPPATH/$Title-0.mpa" | $OGGENC $OGGENCPAR -a "$3" -t "$2" -d "$4" -o "$TARGET/$Title/$3 - $2.ogg" -

rm "$TEMPPATH/$Title-0.mpa"
