/*
 * Channel-Logo-Wipe-Out - A plugin for the Video Disk Recorder
 *
 */
 
#include <vdr/plugin.h>
#include <vdr/status.h>
#include <vdr/remote.h>
#include <vdr/tools.h>
#include <linux/videodev.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <pthread.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <vdr/osd.h>

//*******************************************************
//*******************************************************
//*******************************************************

static const char *VERSION        = "0.0.1_TKK_0.1";
static const char *DESCRIPTION    = "Channellogo WipeOut";
static const char *MAINMENUENTRY  = "WipeOut Channellogo";

//*******************************************************
//*******************************************************
//*******************************************************

cOsd *osd = NULL;

#define VIDEO_DEVICE "/dev/video0"
pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
static unsigned char *map;

int endthread_t1;
int endthread_t2;

int Active_ChannelNumber;
char *Active_ChannelName;

int cap_resol_x = 719;
int cap_resol_y = 575;

#define BLOCKSIZEX 20
#define BLOCKSIZEY 20
#define DELTATHRESHOLD 10
#define CNTTRESHOLD (BLOCKSIZEX*BLOCKSIZEY*1/2)

const cFont *font = cFont::GetFont(fontOsd);

int xp = 0,yp = 0;

// logo position
int logo_x1;
int logo_y1;
int logo_x2;  
int logo_y2;

// drawing area data
int offset_x = 0;
int offset_y = 0;
int size_x = 719;
int size_y = 575;
//*******************************************************
//*******************************************************
//*******************************************************
class V4L_Capture {
	private:
    struct video_mmap v_map;
    void *map;
    int fd;
	public:
    V4L_Capture(int width, int height);
    ~V4L_Capture();
    void *capture();
};

//*******************************************************
//*******************************************************
//*******************************************************
class cProcessStatus : public cStatus {
	private:
		cPlugin* parent;
	protected:
	  virtual void ChannelSwitch(const cDevice *Device, int ChannelNumber);
	public:
		cProcessStatus(cPlugin* plugin);
};

//*******************************************************
//*******************************************************
//*******************************************************
class cProcess : public cOsdObject {
	private:	  
	  int x, y;    
	  tColor color;
	  pthread_t t2;  	
	public:
	  cProcess(void);
	  ~cProcess();
	  void SwitchtoChannel(int channelnumber);
	  virtual void Show(void);
	  virtual eOSState ProcessKey(eKeys Key);
};

//*******************************************************
//*******************************************************
//*******************************************************
class cPluginLogowipeout : public cPlugin {
	private:
	  // Add any member variables or functions you may need here.  
	  pthread_t t1;
	  cProcessStatus *ProcessStatus;
	public:
	  cPluginLogowipeout(void);
	  virtual ~cPluginLogowipeout();
	  virtual const char *Version(void) { return VERSION; }
	  virtual const char *Description(void) { return DESCRIPTION; }
	  virtual const char *CommandLineHelp(void);
	  virtual bool ProcessArgs(int argc, char *argv[]);
	  virtual bool Start(void);  
	  virtual void Stop(void);  
	  virtual void Housekeeping(void);
	  virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
	  virtual cOsdObject *MainMenuAction(void);
	  virtual cMenuSetupPage *SetupMenu(void);
	  virtual bool SetupParse(const char *Name, const char *Value);
	  virtual void MainThreadHook(void);
};

//*******************************************************
//*******************************************************
//*******************************************************
V4L_Capture::V4L_Capture(int width, int height) {
    int res;
    if((fd=open(VIDEO_DEVICE,O_RDWR))==-1) {
        perror(NULL);       
        return;
    }

    struct video_capability cap;
    ioctl(fd,VIDIOCGCAP,&cap);
    printf("Device: %s\nchannels: %d\nmax resolution: %dx%d\n",cap.name,cap.channels,cap.maxwidth,cap.maxheight);

    struct video_window win;
    res = ioctl(fd,VIDIOCGWIN,&win);
    printf ("get CGWIN %d\n",res);
    win.x=win.y=win.chromakey=0;
    win.width=width;
    win.height=height;
    win.flags=0;
    res = ioctl(fd,VIDIOCSWIN,&win);
    printf ("set CGWIN %d\n",res);

    struct video_picture pic;
    res = ioctl(fd,VIDIOCGPICT,&pic);
    printf ("get CGPICT %d\n",res);

    pic.depth=24;
    pic.palette=VIDEO_PALETTE_RGB24;
    pic.brightness=pic.hue=pic.colour=pic.contrast=pic.whiteness=32767;
    res = ioctl(fd,VIDIOCSPICT,&pic);
    printf ("set CSPICT %d\n",res);

    struct video_mbuf m_buf;
    res = ioctl(fd,VIDIOCGMBUF,&m_buf);
    printf ("get CGMBUF %d size: %d\n",res, m_buf.size);

    map=mmap(0,m_buf.size,PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);

    v_map.frame=0;
    v_map.width=width;
    v_map.height=height;
    v_map.format=VIDEO_PALETTE_RGB24;
    printf("Set up complete: %d\n",map);
}

//*******************************************************
//*******************************************************
//*******************************************************
V4L_Capture::~V4L_Capture() {
    close(fd);
}

//*******************************************************
//*******************************************************
//*******************************************************
void *V4L_Capture::capture() {
    ioctl(fd,VIDIOCMCAPTURE,&v_map);
    ioctl(fd,VIDIOCSYNC,&v_map);
    return map;
}

//*******************************************************
//*******************************************************
//*******************************************************
void* Capture_thread(void* s)
{	
    V4L_Capture cap(cap_resol_x, cap_resol_y);
    
    while (!endthread_t1) {			
			pthread_mutex_lock(&mutex);
    	map = (unsigned char *)cap.capture();        //capture screen
    	pthread_mutex_unlock(&mutex);
      usleep(1);  
		}
		
		return NULL;
}

//*******************************************************
//*******************************************************
//*******************************************************
void* WipeOutThread(void* s)
{	
		char txt[20];
		int red, green, blue;
		int x, y;
		
		while (!endthread_t2) {			
				
	  	if (osd) {      	  						
				for (y = logo_y1; y < logo_y2; y++) {  		
			  	for (x = logo_x1; x < logo_x2; x++) {  					  					  					  	
			  		
			  		// add copy-region offset
			  		if (logo_y1 > 250)
			 				y -= ((logo_y2-logo_y1) + 5);
			 			else
			 				y += ((logo_y2-logo_y1) + 5);			 				
//			 			x-=7;						
			 			x-=3;						
			 			
			 			// readout copy-region
						pthread_mutex_lock(&mutex);
				  	blue =*(map + 0 + 3*x + 3*cap_resol_x*y);
				    green = *(map + 1 + 3*x + 3*cap_resol_x*y);
				    red = *(map + 2 + 3*x + 3*cap_resol_x*y);
				    pthread_mutex_unlock(&mutex);
				    
				    // undo offset
						if (logo_y1 > 250)
			 				y += ((logo_y2-logo_y1) + 5);
			 			else
			 				y -= ((logo_y2-logo_y1) + 5);
//						x+=7;
						x+=3;
						
						// quant.values
					  red = 42.5*(int)(red/42.5);
						green = 42.5*(int)(green/42.5);
						blue = 42.5*(int)(blue/42.5);						
						
						if (red>255) red = 255;
						if (green>255) green = 255;
						if (blue>255) blue = 255;
													
						// overwrite logo
						osd->DrawPixel(x, y, 0xFF000000+(red<<16)+(green<<8)+(blue));
					}
			 	}

				// logo search help
				if (xp + yp > 0)
				{			 	
			 		sprintf(txt,"X:%d, Y:%d",xp,yp);     
     			osd->DrawText(360,50, txt, 0x2FFFFFFF, clrTransparent, font);
     			osd->DrawPixel(xp, yp, clrYellow);          		
     		
					osd->Flush(); 			 
				
					osd->DrawText(360,50, txt, clrTransparent, clrTransparent, font);
     			osd->DrawPixel(xp, yp, clrTransparent);
     		 } 
     		 else
     		 	osd->Flush(); 			 
	  	}    
	  		  
	  usleep(1);
	 }
	 
	 return NULL;
}

//*******************************************************
//*******************************************************
//*******************************************************
cProcess::cProcess(void)
{
  osd = NULL;  
  x = y = 50;
  color = clrRed;
}

//*******************************************************
//*******************************************************
//*******************************************************
cProcess::~cProcess()
{	
//	printf("beende wipeout-thread\n");  	    
  endthread_t2 = 1;
  pthread_join(t2, NULL);
  
  delete osd;
}

//*******************************************************
//*******************************************************
//*******************************************************
void cProcess::Show(void)
{		  
  osd = cOsdProvider::NewOsd(0, 0);
  
  if (osd) {  	
     tArea Area = { logo_x1, logo_y1, logo_x2, logo_y2, 8};
     osd->SetAreas(&Area, 1);          
     osd->DrawRectangle(logo_x1,logo_y1,logo_x2,logo_y2, clrTransparent);     
//     osd->DrawRectangle(logo_x1,logo_y1,logo_x2,logo_y2, 8);     
     osd->Flush();     
  }  
  
//  printf("starte wipeout-thread\n");
  endthread_t2 = 0;
	pthread_create (&t2, NULL, WipeOutThread, NULL);	
}

//*******************************************************
//*******************************************************
//*******************************************************
void cProcess::SwitchtoChannel(int channelnumber)
{
	const cChannel* newChannel = Channels.GetByNumber(channelnumber);
	cDevice::PrimaryDevice()->SwitchChannel(newChannel, true);
}

//*******************************************************
//*******************************************************
//*******************************************************
eOSState cProcess::ProcessKey(eKeys Key)
{
  eOSState state = cOsdObject::ProcessKey(Key);  
  
  if (state == osUnknown) {
    switch (Key & ~k_Repeat) {     	
    	case kUp:			SwitchtoChannel(Active_ChannelNumber+1);
     	 							return osEnd;
      case kDown:		if (Active_ChannelNumber > 0)
       									SwitchtoChannel(Active_ChannelNumber-1);
       							return osEnd;
/*
      case k2:      if (yp > offset_y)   yp--; break;
      case k8:   	  if (yp < size_y + offset_y) yp++; break;
      case k4:   	  if (xp > offset_x)   xp--; break;
      case k6:			if (xp < size_x + offset_x)  xp++; break;
*/
      case kOk:     return osEnd;
      case kBack:     return osEnd;
      //default: 			return state;      			 
    }       
  }    
  state = osContinue;
  return state;
}

//*******************************************************
//*******************************************************
//*******************************************************
cProcessStatus::cProcessStatus(cPlugin* plugin)
{
	parent = plugin;
}
	
//*******************************************************
//*******************************************************
//*******************************************************
void cProcessStatus::ChannelSwitch(const cDevice *Device, int ChannelNumber)
{
  const cChannel* Channel = Channels.GetByNumber(Device->CurrentChannel());     
  tChannelID test = Channel->GetChannelID();
  cString st = test.ToString();  
  
  //dirty i know ;-)
  char chID[30];
	strcpy(chID,st);
	
  Active_ChannelNumber = ChannelNumber;
    
//	printf("\nSwitched to channel number: %d\		channel name: %s\n", ChannelNumber, Channel->ShortName(true));  	
//	printf("ChannelID: %s\n", chID);
	logo_x1 = 0;
	logo_y1 = 0;
	logo_x2 = 0;  
	logo_y2 = 0;
	
	// ORF 1, ORF 2
	if ((strcmp(chID, "S19.2E-1-1117-13001") == 0) || (strcmp(chID, "S19.2E-1-1117-13008") == 0))
	{		
		logo_x1 = 577;
		logo_y1 = 37;
		logo_x2 = 665;  
		logo_y2 = 68;
		cRemote::CallPlugin(parent->Name());
	}

	// ATV
	if (strcmp(chID, "S19.2E-1-1117-13012") == 0)
	{						
		logo_x1 = 597;
		logo_y1 = 47;
		logo_x2 = 660;  
		logo_y2 = 71;
		cRemote::CallPlugin(parent->Name());
	}
	
	
	// Kabel 1
	if (strcmp(chID, "S19.2E-133-33-899") == 0)
	{						
		logo_x1 = 615;
		logo_y1 = 35;
		logo_x2 = 660;  
		logo_y2 = 90;
		cRemote::CallPlugin(parent->Name());
	}
	
	
	// RTL
	if (strcmp(chID, "S19.2E-1-1089-12003") == 0)
	{						
		logo_x1 = 55;
		logo_y1 = 41;
		logo_x2 = 150;  
		logo_y2 = 66;
		cRemote::CallPlugin(parent->Name());
	}
	
	// VOX
	if (strcmp(chID, "S19.2E-1-1089-12060") == 0)
	{						
		logo_x1 = 55;
		logo_y1 = 43;
		logo_x2 = 128;  
		logo_y2 = 70;
		cRemote::CallPlugin(parent->Name());
	}
	
	// Bayerisches FS
	if (strcmp(chID, "S19.2E-1-1101-28107") == 0)
	{
		logo_x1 = 48;
		logo_y1 = 30;
		logo_x2 = 103;  
		logo_y2 = 95;
		cRemote::CallPlugin(parent->Name());
	}
	
	// 3sat
	if (strcmp(chID, "S19.2E-1-1079-28007") == 0)
	{		
		logo_x1 = 52;
		logo_y1 = 39;
		logo_x2 = 120;  
		logo_y2 = 66;
		cRemote::CallPlugin(parent->Name());
	}
	
	// ProSieben
	if (strcmp(chID, "S19.2E-133-33-898") == 0)
	{		
		logo_x1 = 633;
		logo_y1 = 47;
		logo_x2 = 664;  
		logo_y2 = 79;		
		cRemote::CallPlugin(parent->Name());
	}
	
	// Das Erste
	if (strcmp(chID, "S19.2E-1-1101-28106") == 0)
	{		
		logo_x1 = 636;
		logo_y1 = 41;
		logo_x2 = 666;  
		logo_y2 = 75;
		cRemote::CallPlugin(parent->Name());
	}
	
	// ZDF
	if (strcmp(chID, "S19.2E-1-1079-28006") == 0)
	{		
		logo_x1 = 51;
		logo_y1 = 35;
		logo_x2 = 118;  
		logo_y2 = 77;
		cRemote::CallPlugin(parent->Name());
	}
	
	// RTL2
	if (strcmp(chID, "S19.2E-1-1089-12020") == 0)
	{		
		logo_x1 = 596;
		logo_y1 = 452;
		logo_x2 = 649;  
		logo_y2 = 519;
		cRemote::CallPlugin(parent->Name());
	}
	
	// SAT.1
	if (strcmp(chID, "S19.2E-133-33-46") == 0)
	{		
		logo_x1 = 573;
		logo_y1 = 50;
		logo_x2 = 670;  
		logo_y2 = 80;
		cRemote::CallPlugin(parent->Name());
	}
	
	// ZDFinfokanal
	if (strcmp(chID, "S19.2E-1-1079-28011") == 0)
	{		
		logo_x1 = 53;
		logo_y1 = 35;
		logo_x2 = 164;  
		logo_y2 = 77;
		cRemote::CallPlugin(parent->Name());
	}	

	// Das Vierte
	if (strcmp(chID, "S19.2E-133-5-1793") == 0)
	{		
		logo_x1 = 58;
		logo_y1 = 38;
		logo_x2 = 182;  
		logo_y2 = 81;
		cRemote::CallPlugin(parent->Name());
	}	
}

//*******************************************************
//*******************************************************
//*******************************************************
cPluginLogowipeout::cPluginLogowipeout(void)
{
  // Initialize any member variables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!    
}

//*******************************************************
//*******************************************************
//*******************************************************
cPluginLogowipeout::~cPluginLogowipeout()
{
  // Clean up after yourself!
  delete ProcessStatus;
}

//*******************************************************
//*******************************************************
//*******************************************************
const char *cPluginLogowipeout::CommandLineHelp(void)
{
  // Return a string that describes all known command line options.
  return NULL;
}

//*******************************************************
//*******************************************************
//*******************************************************
bool cPluginLogowipeout::ProcessArgs(int argc, char *argv[])
{
  // Implement command line argument processing here if applicable.
  return true;
}

//*******************************************************
//*******************************************************
//*******************************************************
bool cPluginLogowipeout::Start(void)
{
  // Start any background activities the plugin shall perform.        
  
//  printf("starte capture-thread\n");
  endthread_t1 = 0;  
	pthread_create (&t1, NULL, Capture_thread, NULL);	
		
//	printf("channelstatus supervision started\n");
  ProcessStatus = new cProcessStatus(this);
  
  return true;
}

//*******************************************************
//*******************************************************
//*******************************************************
void cPluginLogowipeout::Stop(void)
{
  // Stop any background activities the plugin shall perform.
//  printf("beende capture-thread\n");  
  endthread_t1 = 1;
  pthread_join(t1, NULL);
}

//*******************************************************
//*******************************************************
//*******************************************************
void cPluginLogowipeout::Housekeeping(void)
{
  // Perform any cleanup or other regular tasks.	
}

//*******************************************************
//*******************************************************
//*******************************************************
cOsdObject *cPluginLogowipeout::MainMenuAction(void)
{
  // Perform the action when selected from the main VDR menu.
  return new cProcess;
}

//*******************************************************
//*******************************************************
//*******************************************************
void cPluginLogowipeout::MainThreadHook(void)
{
  // Perform actions in the context of the main program thread.
  // WARNING: Use with great care - see PLUGINS.html!  
}

//*******************************************************
//*******************************************************
//*******************************************************
cMenuSetupPage *cPluginLogowipeout::SetupMenu(void)
{
  // Return a setup menu in case the plugin supports one.
  return NULL;
}

//*******************************************************
//*******************************************************
//*******************************************************
bool cPluginLogowipeout::SetupParse(const char *Name, const char *Value)
{
  // Parse your own setup parameters and store their values.
  return false;
}

VDRPLUGINCREATOR(cPluginLogowipeout); // Don't touch this!
