/*
 * vdr-podcatcher - A plugin for the Linux Video Disk Recorder
 * Copyright (c) 2006 Tobias Grimm <vdr@e-tobi.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: decodeaction_test.cc 3219 2006-04-16 15:06:35Z svntobi $
 *
 */

#include <cxxtest/TestSuite.h>
#include "decodeaction.h"
#include "circularpespacketbuffer.h"
#include "pcmdatasource.h"
#include "pesaudiopacket.h"

using namespace std;

class PcmDataSourceStub: public IPcmDataSource
{
    public:
        bool eof;

    public:
        void FillPesAudioPacket(PesAudioPacket& packet)
        {
            packet.AddSample(0, 0);
        }

        bool Eof()
        {
            return eof;
        }
};

class DecodeActionTest: public CxxTest::TestSuite
{
    public:
        void TestAbortOnEof()
        {
            PcmDataSourceStub* pcmDataSourceStub = new PcmDataSourceStub();
            ref_ptr<IPcmDataSource> source(pcmDataSourceStub);
            ref_ptr<IPesPacketBuffer> buffer(new CircularPesPacketBuffer(4));
            pcmDataSourceStub->eof = true;
            DecodeAction action(source, buffer);

            TS_ASSERT(!action.Action());
            TS_ASSERT(NULL == buffer->GetPesPacket().get());
        }

        void TestCreatePacketAndPutIntoBuffer()
        {
            PcmDataSourceStub* pcmDataSourceStub = new PcmDataSourceStub();
            ref_ptr<IPcmDataSource> source(pcmDataSourceStub);
            ref_ptr<IPesPacketBuffer> buffer(new CircularPesPacketBuffer(4));
            pcmDataSourceStub->eof = false;
            DecodeAction action(source, buffer);

            TS_ASSERT(action.Action());
            ref_ptr<PesAudioPacket> packet = buffer->GetPesPacket();
            TS_ASSERT(NULL != packet.get());
            TS_ASSERT_EQUALS(16+2+2, packet->GetBufferSize());
        }

        void TestOnEofAtSourceSignalEofToBuffer()
        {
            PcmDataSourceStub* pcmDataSourceStub = new PcmDataSourceStub();
            ref_ptr<IPcmDataSource> source(pcmDataSourceStub);
            ref_ptr<IPesPacketBuffer> buffer(new CircularPesPacketBuffer(4));
            DecodeAction action(source, buffer);

            pcmDataSourceStub->eof = false;
            action.Action();
            TS_ASSERT(!buffer->Eof());

            pcmDataSourceStub->eof = true;
            action.Action();
            TS_ASSERT(buffer->Eof());
        }
};
