/*
 * vdr-podcatcher - A plugin for the Linux Video Disk Recorder
 * Copyright (c) 2006 Tobias Grimm <vdr@e-tobi.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: downloadqueue_test.cc 2511 2006-01-28 14:05:09Z svntobi $
 *
 */

#include <cxxtest/TestSuite.h>
#include "downloadqueue.h"

class DownloadQueueTest: public CxxTest::TestSuite
{
    private:
        DownloadQueue* queue;

    public:
        void setUp()
        {
            queue = new DownloadQueue();
        }

        void tearDown()
        {
            delete queue;
        }

        void TestEmptyQueue()
        {
            TS_ASSERT(NULL == queue->GetNextDownload().get());
        }

        void TestAddDownload()
        {
            ref_ptr<Download> download(new Download("url"));
            queue->AddDownload(download);
            TS_ASSERT(NULL != queue->GetNextDownload().get());
            TS_ASSERT_EQUALS("url", queue->GetNextDownload()->GetUrl());
        }

        void TestRemoveDownloadNotInQueue()
        {
            ref_ptr<Download> download1(new Download("url1"));
            ref_ptr<Download> download2(new Download("url2"));

            queue->AddDownload(download1);
            TS_ASSERT(NULL != queue->GetNextDownload().get());

            queue->RemoveDownload(download2);
            TS_ASSERT(NULL != queue->GetNextDownload().get());
        }

        void TestRemoveDownload()
        {
            ref_ptr<Download> download1(new Download("url1"));
            ref_ptr<Download> download2(new Download("url2"));

            queue->AddDownload(download1);
            queue->AddDownload(download2);

            queue->RemoveDownload(download1);
            TS_ASSERT_EQUALS("url2", queue->GetNextDownload()->GetUrl());

            queue->RemoveDownload(download2);
            TS_ASSERT(NULL == queue->GetNextDownload().get());
        }

        void TestDontAddDuplicates()
        {
            ref_ptr<Download> download1(new Download("url"));
            ref_ptr<Download> download2(new Download("url"));

            queue->AddDownload(download1);
            queue->AddDownload(download2);

            queue->RemoveDownload(download1);

            TS_ASSERT(NULL == queue->GetNextDownload().get());
        }

        void TestFifo()
        {
            ref_ptr<Download> download1(new Download("url1"));
            ref_ptr<Download> download2(new Download("url2"));

            queue->AddDownload(download1);
            queue->AddDownload(download2);
            TS_ASSERT_EQUALS("url1", queue->GetNextDownload()->GetUrl());

            queue->RemoveDownload(download1);
            TS_ASSERT_EQUALS("url2", queue->GetNextDownload()->GetUrl());
        }
};
