/*
 * vdr-podcatcher - A plugin for the Linux Video Disk Recorder
 * Copyright (c) 2006 Tobias Grimm <vdr@e-tobi.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: playbackaction_test.cc 3342 2006-04-24 21:30:53Z svntobi $
 *
 */

#include <cxxtest/TestSuite.h>
#include "pesplayer.h"
#include "circularpespacketbuffer.h"
#include "playbackaction.h"
#include "stringmessagemock.h"
#include "pesaudiopacket.h"

using namespace std;

class PesPlayerMock: public StringMessageMock, public IPesPlayer
{
    public:
        //<IPesPlayer>
        void PlayPes(PesAudioPacket& packet)
        {
            AddMethod("PlayPes", MakeString(&packet));
        }

        bool IsPlaying()
        {
            return false;
        }

        unsigned int GetTimeIndex()
        {
            return 0;
        }

        int GetTotalTime()
        {
            return 0;
        }
        //</IPesPlayer>
};

class PlaybackActionTest: public CxxTest::TestSuite
{
    public:
        void TestBufferIsEmpty()
        {
            ref_ptr<IPesPacketBuffer> buffer(new CircularPesPacketBuffer(1));
            PesPlayerMock player;
            PlaybackAction action(buffer, player);

            TS_ASSERT(action.Action());
            player.Verify();
        }

        void TestPlayPacketFromBuffer()
        {
            ref_ptr<IPesPacketBuffer> buffer(new CircularPesPacketBuffer(1));
            PesPlayerMock player;
            PlaybackAction action(buffer, player);

            ref_ptr<PesAudioPacket> packet(new PesAudioPacket(20));
            buffer->PutPesPacket(packet);
            buffer->SetEof(false);

            player.ExpectMethod("PlayPes", player.MakeString(packet.get()));
            TS_ASSERT(action.Action());
            player.Verify();
        }

        void TestAbortPlaybackIfEofReachedAndBufferEmpty()
        {
            ref_ptr<IPesPacketBuffer> buffer(new CircularPesPacketBuffer(1));
            PesPlayerMock player;
            PlaybackAction action(buffer, player);

            buffer->SetEof(true);

            TS_ASSERT(!action.Action());
        }

        void TestDontAbortPlaybackIfEofReachedButBufferNotEmpty()
        {
            ref_ptr<IPesPacketBuffer> buffer(new CircularPesPacketBuffer(1));
            PesPlayerMock player;
            PlaybackAction action(buffer, player);

            ref_ptr<PesAudioPacket> packet(new PesAudioPacket(20));
            buffer->PutPesPacket(packet);
            buffer->SetEof(true);

            TS_ASSERT(action.Action());
        }
};
