/*
 * vdr-podcatcher - A plugin for the Linux Video Disk Recorder
 * Copyright (c) 2006 Tobias Grimm <vdr@e-tobi.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: playbackcontroller_test.cc 3342 2006-04-24 21:30:53Z svntobi $
 *
 */

#include <cxxtest/TestSuite.h>
#include "playbackcontroller.h"
#include "circularpespacketbuffer.h"
#include "pesaudiopacket.h"
#include "threadmock.h"

using namespace std;

class PlaybackControllerTest: public CxxTest::TestSuite
{
    private:
        stringstream* mp3StreamPtr;
        ThreadMock* playbackThreadPtr;
        ref_ptr<istream> mp3Stream;
        ref_ptr<IThread> playbackThread;
        ref_ptr<IPesPacketBuffer> pesBuffer;
        ref_ptr<PlaybackController> controller;

    public:
        void setUp()
        {
            mp3Stream = mp3StreamPtr = new stringstream();
            playbackThread = playbackThreadPtr = new ThreadMock();;
            pesBuffer = new CircularPesPacketBuffer(10);
            controller = new PlaybackController(mp3Stream, pesBuffer,
              playbackThread);
        }

        void TestPlaybackFinishedWhenMp3StreamAndBufferAreEmpty()
        {
            TS_ASSERT(controller->IsPlaybackFinished());
        }

        void TestPlaybackNotFinishedWhenBytesLeftInMp3Stream()
        {
            *mp3StreamPtr <<  "0123456789";

            TS_ASSERT(!controller->IsPlaybackFinished());
        }

        void TestPlaybackFinishedWhenMp3StreamAtEofAndBufferIsEmpty()
        {
            *mp3StreamPtr <<  "0123456789";
            mp3Stream->ignore(10);

            TS_ASSERT(controller->IsPlaybackFinished());
        }

        void TestPlaybackNotFinishedWhenBufferNotEmpty()
        {
            pesBuffer->PutPesPacket(ref_ptr<PesAudioPacket>(
              new PesAudioPacket(100)));

            TS_ASSERT(!controller->IsPlaybackFinished());
        }

        void TestIsPlaying()
        {
            playbackThreadPtr->Start();
            TS_ASSERT(controller->IsPlaying());

            playbackThreadPtr->Stop();
            TS_ASSERT(!controller->IsPlaying());
        }

        void TestPauseStopsPlaybackThread()
        {
            playbackThreadPtr->Start();
            playbackThreadPtr->ResetMock();
            playbackThreadPtr->ExpectMethod("Stop");

            controller->Pause();

            playbackThreadPtr->Verify();
        }

        void TestPauseStartsPlaybackThread()
        {
            playbackThreadPtr->ExpectMethod("Start");
            controller->Pause();
            playbackThreadPtr->Verify();
        }
};
