/*
 * vdr-podcatcher - A plugin for the Linux Video Disk Recorder
 * Copyright (c) 2006 Tobias Grimm <vdr@e-tobi.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: threadsafedownloadpool_test.cc 3219 2006-04-16 15:06:35Z svntobi $
 *
 */

#include <cxxtest/TestSuite.h>
#include "stringmessagemock.h"
#include "threadsafedownloadpool.h"
#include "downloadpoolmock.h"

using namespace std;

namespace
{
    class CriticalSectionMock: public StringMessageMock, public ICriticalSection
    {
        public:
            CriticalSectionMock(StringMessageMock& parent): StringMessageMock(parent) {}
            // <ICriticalSection>
            void Enter()
            {
                AddMethod("Enter");
            }

            void Leave()
            {
                AddMethod("Leave");
            }
            // </ICriticalSection>
    };

    class ThreadSafeDownloadPoolTest: public CxxTest::TestSuite
    {
        private:
            StringMessageMock combinedMock;
            DownloadPoolMock* downloadPool;
            ICriticalSection* criticalSection;
            ThreadSafeDownloadPool* queue;

        public:
            void setUp()
            {
                combinedMock.ResetMock();
                downloadPool = new DownloadPoolMock(combinedMock);
                criticalSection = new CriticalSectionMock(combinedMock);
                queue = new ThreadSafeDownloadPool(ref_ptr<IDownloadPool>(downloadPool),
                   ref_ptr<ICriticalSection>(criticalSection));
            }

            void tearDown()
            {
                delete queue;
            }

            void TestAddDownload()
            {
                ref_ptr<Download> download(new Download("url"));

                combinedMock.ExpectMethod("Enter");
                combinedMock.ExpectMethod("AddDownload", "url");
                combinedMock.ExpectMethod("Leave");

                queue->AddDownload(download);

                combinedMock.Verify();
            }

            void TestGetNextDownload()
            {
                ref_ptr<Download> download(new Download("url"));
                downloadPool->returnedDownload = download;

                combinedMock.ExpectMethod("Enter");
                combinedMock.ExpectMethod("GetNextDownload");
                combinedMock.ExpectMethod("Leave");

                TS_ASSERT_EQUALS("url", queue->GetNextDownload()->GetUrl());

                combinedMock.Verify();
            }

            void TestRemoveDownload()
            {
                ref_ptr<Download> download(new Download("foo"));

                combinedMock.ExpectMethod("Enter");
                combinedMock.ExpectMethod("RemoveDownload", "foo");
                combinedMock.ExpectMethod("Leave");

                queue->RemoveDownload(download);

                combinedMock.Verify();
            }

            void TestGetDownloadByUrl()
            {
                ref_ptr<Download> download(new Download("url"));
                downloadPool->returnedDownload = download;

                combinedMock.ExpectMethod("Enter");
                combinedMock.ExpectMethod("GetDownloadByUrl", "url");
                combinedMock.ExpectMethod("Leave");

                TS_ASSERT_EQUALS("url",
                  queue->GetDownloadByUrl("url")->GetUrl());

                combinedMock.Verify();
            }
    };
};
